#ifndef MyScalarBarActor_H
#define MyScalarBarActor_H

#include <vtkScalarBarActor.h>

class MyScalarBarActor : public vtkScalarBarActor
{
public:
  vtkTypeMacro(MyScalarBarActor,vtkActor2D);

  // Description:
  // Instantiate object with 64 maximum colors; 5 labels; %%-#6.3g label
  // format, no title, and vertical orientation. The initial scalar bar
  // size is (0.05 x 0.8) of the viewport size.
  static MyScalarBarActor* New();

  void PrintSelf(ostream& os, vtkIndent indent) {}
  void PrintHeader(ostream& os, vtkIndent indent) {}
  void PrintTrailer(ostream& os, vtkIndent indent) {}

  // Description:
  // Draw the scalar bar and annotation text to the screen.
  int RenderOverlay(vtkViewport* viewport);

protected:
  MyScalarBarActor();
  ~MyScalarBarActor();

  // Description:
  // Called from within \a RenderOpaqueGeometry when the internal state
  // members need to be updated before rendering.
  //
  // This method invokes many virtual methods that first lay out the
  // scalar bar and then use the layout to position actors and create
  // datasets used to represent the scalar bar.
  // Specifically, it invokes: FreeLayoutStorage, ComputeFrame,
  // ComputeScalarBarThickness, LayoutNanSwatch, PrepareTitleText,
  // LayoutTitle, ComputeScalarBarLength, LayoutTicks, and LayoutAnnotations
  // to perform the layout step.
  // Then, it invokes ConfigureAnnotations, ConfigureFrame,
  // ConfigureScalarBar, ConfigureTitle, ConfigureTicks, and
  // ConfigureNanSwatch to create and position actors used to render
  // portions of the scalar bar.
  //
  // By overriding one or more of these virtual methods, subclasses
  // may change the appearance of the scalar bar.
  //
  // In the layout phase, text actors must have their text properties
  // and input strings updated, but the position of the actors should
  // not be set or relied upon as subsequent layout steps may alter
  // their placement.
  virtual void RebuildLayout(vtkViewport* viewport);

  // Description:
  // Free internal storage used by the previous layout.
  virtual void FreeLayoutStorage();

  // Description:
  // If the scalar bar should be inset into a frame or rendered with a
  // solid background, this method will inset the outermost scalar bar
  // rectangle by a small amount to avoid having the scalar bar
  // illustration overlap any edges.
  //
  // This method must set the frame coordinates (this->P->Frame).
  virtual void ComputeFrame();

  // Description:
  // Determine how thick the scalar bar should be (on an axis perpendicular
  // to the direction in which scalar values vary).
  //
  // This method must set the scalar bar thickness
  // (this->P->ScalarBarBox.Size[0]).
  // It may depend on layout performed by ComputeFrame
  // (i.e., the frame coordinates in this->P->Frame).
  virtual void ComputeScalarBarThickness();

  // Description:
  // Determine the size of the NaN swatch if it is to be rendered.
  //
  // This method must set this->P->NanSwatchSize and this->P->NanBox.
  // It may depend on layout performed by ComputeScalarBarThickness.
  virtual void LayoutNanSwatch();

  // Description:
  // Set the title actor's input to the latest title (and subtitle) text.
  virtual void PrepareTitleText();

  // Description:
  // Determine the position and size of the scalar bar title box.
  //
  // This method must set this->P->TitleBox
  // It may depend on layout performed by LayoutNanSwatch.
  // If useTickBox is true, it should increase the target area
  // for the label to touch the tick box. It is called in this
  // way when the tick labels are small due to constraints other
  // than the title box.
  virtual void LayoutTitle();

  // Description:
  // Determine how long the scalar bar should be (on an axis parallel
  // to the direction in which scalar values vary).
  //
  // This method must set this->P->ScalarBarBox.Size[1] and should
  // estimate this->P->ScalarBarBox.Posn.
  // It may depend on layout performed by LayoutTitle.
  virtual void ComputeScalarBarLength();

  // Description:
  // Determine the size and placement of any tick marks to be rendered.
  //
  // This method must set this->P->TickBox.
  // It may depend on layout performed by ComputeScalarBarLength.
  //
  // The default implementation creates exactly this->NumberOfLabels
  // tick marks, uniformly spaced on a linear or logarithmic scale.
  virtual void LayoutTicks();

  // Description:
  // This method must lay out annotation text and leader lines so
  // they do not overlap.
  //
  // This method must set this->P->AnnotationAnchors.
  // It may depend on layout performed by LayoutTicks.
  virtual void LayoutAnnotations();

  // Description:
  // Generate/configure the annotation labels using the laid-out geometry.
  virtual void ConfigureAnnotations();

  // Description:
  // Generate/configure the representation of the frame from laid-out geometry.
  virtual void ConfigureFrame();

  // Description:
  // For debugging, add placement boxes to the frame polydata.
  virtual void DrawBoxes();

  // Description:
  // Generate/configure the scalar bar representation from laid-out geometry.
  virtual void ConfigureScalarBar();

  // Description:
  // Generate/configure the title actor using the laid-out geometry.
  virtual void ConfigureTitle();

  // Description:
  // Generate/configure the tick-mark actors using the laid-out geometry.
  virtual void ConfigureTicks();

  // Description:
  // Generate/configure the NaN swatch using the laid-out geometry.
  //
  // Currently the NaN swatch is rendered by the same actor as the scalar bar.
  // This may change in the future.
  virtual void ConfigureNanSwatch();

  // Description:
  // Subclasses may override this method to alter this->P->Labels, allowing
  // the addition and removal of annotations. The member maps viewport coordinates
  // along the long axis of the scalar bar to text (which may include MathText;
  // see vtkTextActor). It is a single-valued map, so you must perturb
  // the coordinate if you wish multiple labels to annotate the same position.
  // Each entry in this->P->Labels must have a matching entry in this->P->LabelColors.
  virtual void EditAnnotations() { }

  // Description:
  // Compute the best size for the legend title.
  //
  // This guarantees that the title will fit within the frame defined by Position and Position2.
  virtual void SizeTitle(double* titleSize, int* size, vtkViewport* viewport);

private:
  MyScalarBarActor(const MyScalarBarActor&);  // Not implemented.
  void operator=(const MyScalarBarActor&);  // Not implemented.
};

#endif // MyScalarBarActor_H
